@@ -19,14 +19,6 @@ class DateTimeError(RuntimeError):
     pass
 
 
-def get_env_variable(var):
-    try:
-        return os.environ[var]
-    except KeyError:
-        raise Exception(f'must set "{var}" environment variable')
-
-
-OCFL_ROOT = get_env_variable('OCFL_ROOT')
 RELS_INT_NS = {'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#', 'ns1': 'info:fedora/fedora-system:def/model#'}
 DNG_MIMETYPE = 'image/x-adobe-dng'
 JS_MIMETYPE = 'application/javascript'
@@ -38,7 +30,11 @@ mimetypes.add_type(MKV_MIMETYPE, '.mkv', strict=False)
 
 
 def get_mimetype_from_filename(filename):
-    if filename.endswith('.tei.xml') or filename.endswith('.tei'):
+    if filename in ['RELS-INT', 'RELS-EXT', 'MODS', 'DWC']:
+        return 'text/xml'
+    if filename == 'DIGITAL-NEGATIVE':
+        return DNG_MIMETYPE
+    if filename == 'TEI' or filename.lower().endswith('.tei.xml') or filename.lower().endswith('.tei'):
         return 'application/tei+xml'
     if '.' not in filename:
         filename = 'a.%s' % filename
@@ -68,10 +64,10 @@ def get_file_size(full_path):
     return os.stat(full_path).st_size
 
 
-def object_path(pid):
+def object_path(storage_root, pid):
     sha256_checksum = hashlib.sha256(pid.encode('utf8')).hexdigest()
     return os.path.join(
-            OCFL_ROOT,
+            storage_root,
             sha256_checksum[0:3],
             sha256_checksum[3:6],
             sha256_checksum[6:9],
@@ -165,15 +161,15 @@ def utc_datetime_from_string(date_string):
 
 class Object:
 
-    def __init__(self, pid, fallback_to_version_directory=True):
+    def __init__(self, storage_root, pid, fallback_to_version_directory=True, deleted_ok=False):
         self.pid = pid
         self._fallback_to_version_directory = fallback_to_version_directory
-        self.object_path = object_path(self.pid)
+        self.object_path = object_path(storage_root, self.pid)
         if not os.path.exists(self.object_path):
             raise ObjectNotFound()
         self._inventory = self._get_inventory(self.object_path)
         self.head_version = self._inventory['head']
-        if not self._inventory['versions'][self.head_version]['state']:
+        if not (deleted_ok or self._inventory['versions'][self.head_version]['state']):
             raise ObjectDeleted()
         self._files_info = None
         self._rels_int_root = None
@@ -206,14 +202,14 @@ class Object:
         return json.loads(data)
 
     def _get_files_info(self):
-        info = {}
+        files_info = {}
         for checksum, filepaths in self._inventory['versions'][self.head_version]['state'].items():
             for filepath in filepaths:
-                if filepath not in info:
+                if filepath not in files_info:
                     file_info = {
-                            'last_modified': utc_datetime_from_string(self._inventory['versions'][self.head_version]['created']),
+                            'lastModified': utc_datetime_from_string(self._inventory['versions'][self.head_version]['created']),
                             'checksum': checksum,
-                            'checksum_type': 'SHA-512',
+                            'checksumType': 'SHA-512',
                             'state': 'A',
                             'size': get_file_size(os.path.join(self.object_path, self._inventory['manifest'][checksum][0])),
                         }
@@ -222,26 +218,43 @@ class Object:
                         download_filename = filepath
                     mimetype = get_mimetype_from_filename(download_filename)
                     file_info['mimetype'] = mimetype
-                    file_info['download_filename'] = download_filename
-                    info[filepath] = file_info
-        #need to backtrack through versions to get/verify the correct last_modified time
-        #if a file was present with the same checksum in the previous version, then the last_modified time needs to be updated
-        file_handled_mapping = {} #tells us not to update the last_modified time anymore as we keep going back through version history
-        for filepath in info.keys():
+                    file_info['downloadFilename'] = download_filename
+                    files_info[filepath] = file_info
+        #need to backtrack through versions to get/verify the correct lastModified time
+        #if a file was present with the same checksum in the previous version, then the lastModified time needs to be updated
+        file_handled_mapping = {} #tells us not to update the lastModified time anymore as we keep going back through version history
+        for filepath in files_info.keys():
             file_handled_mapping[filepath] = False
         for version_num in Object.reversed_version_numbers(self.head_version)[1:]: #already handled head version
             files_in_this_version = set()
             for checksum, filepaths in self._inventory['versions'][version_num]['state'].items():
                 for filepath in filepaths:
-                    if filepath in info:
-                        files_in_this_version.add(filepath)
-                        if checksum == info[filepath]['checksum'] and not file_handled_mapping[filepath]:
-                            info[filepath]['last_modified'] = utc_datetime_from_string(self._inventory['versions'][version_num]['created'])
+                    files_in_this_version.add(filepath)
+                    if filepath in files_info: #we already saw this file in a newer version - update lastModified if needed
+                        if checksum == files_info[filepath]['checksum'] and not file_handled_mapping[filepath]:
+                            files_info[filepath]['lastModified'] = utc_datetime_from_string(self._inventory['versions'][version_num]['created'])
+                    else:
+                        file_info = {
+                                'lastModified': utc_datetime_from_string(self._inventory['versions'][version_num]['created']),
+                                'checksum': checksum,
+                                'checksumType': 'SHA-512',
+                                'state': 'D',
+                                'size': get_file_size(os.path.join(self.object_path, self._inventory['manifest'][checksum][0])),
+                            }
+                        rels_int_root = self._get_rels_int_root(version=version_num)
+                        download_filename = get_download_filename_from_rels_int(rels_int_root, self.pid, filepath)
+                        if not download_filename:
+                            download_filename = filepath
+                        mimetype = get_mimetype_from_filename(download_filename)
+                        file_info['mimetype'] = mimetype
+                        file_info['downloadFilename'] = download_filename
+                        files_info[filepath] = file_info
+                        file_handled_mapping[filepath] = False
             #if there are any files in the head version, that aren't in this version, mark that we shouldn't update their time anymore
             for filepath in file_handled_mapping:
                 if filepath not in files_in_this_version:
                     file_handled_mapping[filepath] = True
-        return info
+        return files_info
 
     @property
     def created(self):
@@ -251,14 +264,17 @@ class Object:
     def last_modified(self):
         return utc_datetime_from_string(self._inventory['versions'][self.head_version]['created'])
 
+    def _get_rels_int_root(self, version):
+        try:
+            rels_int_path = self.get_path_to_file('RELS-INT', version=version)
+            return load_rels_int(rels_int_path)
+        except FileNotFoundError:
+            pass
+
     @property
     def rels_int_root(self):
         if not self._rels_int_root:
-            try:
-                rels_int_path = self.get_path_to_file('RELS-INT')
-                self._rels_int_root = load_rels_int(rels_int_path)
-            except FileNotFoundError:
-                self._rels_int_root = None
+            self._rels_int_root = self._get_rels_int_root(self.head_version)
         return self._rels_int_root
 
     def get_path_to_file(self, filename, version=None):
@@ -288,8 +304,16 @@ class Object:
                     filenames.add(f)
         return filenames
 
-    def get_files_info(self):
-        #returns file information for *active* files
+    def get_files_info(self, include_deleted=False, fields=None):
+        if not fields:
+            if include_deleted:
+                return {filename: {} for filename in self.all_filenames}
+            else:
+                return {filename: {} for filename in self.filenames}
         if not self._files_info:
             self._files_info = self._get_files_info()
-        return self._files_info
+        files_info = {}
+        for filename, info in self._files_info.items():
+            if info['state'] == 'A' or include_deleted:
+                files_info[filename] = {field: value for field, value in info.items() if field in fields}
+        return files_info
